import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { CarreraPrograma } from './entity/carrera-programa.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { InstitucionCarreraService } from '../institucion-carrera/institucion-carrera.service';
import { ProgramaService } from '../programa/programa.service';

@Injectable()
export class CarreraProgramaService {
  constructor(
    @InjectRepository(CarreraPrograma)
    private repository: Repository<CarreraPrograma>,
    private institucionService: InstitucionService,
    private institucionCarreraService: InstitucionCarreraService,
    private programaService: ProgramaService,
  ) {}

  async create(id_institucion_carrera: number, id_programa: number) {
    const carrera = await this.institucionCarreraService.findById(
      id_institucion_carrera,
    );
    const programa = await this.programaService.findById(id_programa);

    return this.repository
      .findOne({ carrera, programa })
      .then((existeCarretaPrograma) => {
        if (existeCarretaPrograma)
          throw new ConflictException(
            'Ya existe una carrera programa con este nombre, intente con uno diferente.',
          );
        return this.repository.save(
          this.repository.create({
            carrera,
            programa,
          }),
        );
      })
      .then(() => ({ message: 'Se creó correctamente la carrera programa.' }));
  }

  findById(id_carrera_programa: number) {
    return this.repository
      .findOne({ id_carrera_programa })
      .then((carreraPrograma) => {
        if (!carreraPrograma)
          throw new NotFoundException('No existe esta carrera programa.');
        return carreraPrograma;
      });
  }

  findByIdInstitucion(id_institucion: number) {
    return this.institucionService
      .findById(id_institucion)
      .then((institucion) =>
        this.repository.find({
          join: {
            alias: 'cp',
            innerJoinAndSelect: {
              ic: 'cp.carrera',
              c: 'ic.carrera',
              i: 'ic.institucion',
            },
          },
          where: {
            carrera: { institucion },
          },
        }),
      );
  }

  //No estoy seguro de que esto este bien :)
  delete(id_carrera_programa: number) {
    return this.findById(id_carrera_programa)
      .then((carreraPrograma) => this.repository.delete(carreraPrograma))
      .then((_) => ({
        message: 'Se eliminó correctamente la carrera programa.',
      }));
  }
}
