import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { Carrito } from '../../carrito/entity/carrito.entity';
import { EquipoTipoEntrada } from './equipo-tipo-entrada.entity';
import { Motivo } from '../../motivo/entity/motivo.entity';
import { Prestamo } from '../../prestamo/entity/prestamo.entity';
import { EquipoPrograma } from '../../equipo-programa/entity/equipo-programa.entity';
import { Status } from '../../status/entity/status.entity';

@Entity()
export class Equipo {
  @PrimaryGeneratedColumn()
  id_equipo: number;

  @Column({ type: String, nullable: false, length: 2 })
  equipo: string;

  @Column({ type: String, nullable: false, length: 30 })
  marca: string;

  @Column({ type: String, nullable: false, length: 30 })
  modelo: string;

  @Column({ type: String, nullable: false, length: 25 })
  numero_inventario: string;

  @Column({ type: String, nullable: false, length: 25 })
  numero_serie: string;

  @Column({ type: Boolean, nullable: false, default: false })
  prestado: boolean;

  @ManyToOne(() => Carrito, (carrito) => carrito.equipos)
  @JoinColumn({ name: 'id_carrito' })
  carrito: Carrito;

  @ManyToOne(() => Status, (status) => status.equipos)
  @JoinColumn({ name: 'id_status' })
  status: Status;

  @OneToMany(
    () => EquipoTipoEntrada,
    (equipoTipoEntrada) => equipoTipoEntrada.tipoEntrada,
  )
  equipoTipoEntradas: EquipoTipoEntrada[];

  @OneToMany(() => EquipoPrograma, (equipoPrograma) => equipoPrograma.equipo)
  programas: EquipoPrograma[];

  @OneToMany(() => Motivo, (motivo) => motivo.equipo)
  motivos: Motivo[];

  @OneToMany(() => Prestamo, (prestamo) => prestamo.equipo)
  prestamos: Prestamo[];
}
