import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { InstitucionCarrera } from '../../institucion-carrera/entity/institucion-carrera.entity';
import { Prestamo } from '../../prestamo/entity/prestamo.entity';
import { TipoUsuario } from '../../tipo-usuario/entity/tipo-usuario.entity';

@Entity()
export class Usuario {
  @PrimaryGeneratedColumn()
  id_usuario: number;

  @Column({ type: Boolean, nullable: false, default: true })
  activo: boolean;

  @Column({ type: String, nullable: false, length: 60, default: '' })
  correo: string;

  @Column({ type: Boolean, nullable: false, default: false })
  multa: boolean;

  @Column({ type: String, nullable: false, length: 100 })
  nombre: string;

  @Column({ type: String, nullable: true, length: 60, default: null })
  password?: string;

  @Column({ type: String, nullable: true, length: 10, default: null })
  telefono?: string;

  @Column({ type: String, nullable: false, length: 10 })
  usuario: string;

  @ManyToOne(
    () => InstitucionCarrera,
    (institucionCarrera) => institucionCarrera.usuarios,
    { eager: true },
  )
  @JoinColumn({ name: 'id_institucion_carrera' })
  institucionCarrera?: InstitucionCarrera;

  @ManyToOne(() => TipoUsuario, (tipoUsuario) => tipoUsuario.usuarios, {
    eager: true,
  })
  @JoinColumn({ name: 'id_tipo_usuario' })
  tipoUsuario: TipoUsuario;

  @OneToMany(() => Prestamo, (prestamo) => prestamo.usuario)
  prestamos: Prestamo[];
}
