import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Query,
  UseGuards,
} from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ApiBody, ApiOperation, ApiQuery, ApiTags } from '@nestjs/swagger';
import { UsuarioService } from './usuario.service';
import { UsuarioDgaeDto } from './dto/usuario-dgae.dto';
import { UsuarioDgpDto } from './dto/usuario-dgp.dto';
import { UsuarioRegistrarDto } from './dto/usuario-registrar.dto';
import { UsuarioUpdateDto } from './dto/usuario-update.dto';
import { UsuarioUsuariosDto } from './dto/usuario-usuarios.dto';
import { UsuarioDto } from './dto/usuario.dto';

@Controller('usuario')
@ApiTags('usuario')
export class UsuarioController {
  constructor(private usuarioService: UsuarioService) {}

  /* Pendiente por conexión */
  // @Get('dgae')
  // dgae(@Query() query: UsuarioDgaeDto) {
  //   return this.usuarioService.dgaeDgp(query.usuario);
  // }
  // @Get('dgp')
  // dgp(@Query() query: UsuarioDgpDto) {
  //   return this.usuarioService.dgaeDgp(query.usuario);
  // }

  @Post('registrar')
  @ApiOperation({
    description: 'Enpoint que realiza el registro de un usuario.',
  })
  @ApiBody({
    description:
      'Todas la variables a excepción de id_usuario son obligatorias.',
    examples: {
      ejemplo: { value: { id_usuario: 1, telefono: '', correo: '' } },
    },
  })
  registrar(@Body() body: UsuarioRegistrarDto) {
    return this.usuarioService.registrar(body.id_usuario, body.telefono);
  }

  @Put()
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Enpoint que actualiza la información de un usuario.',
  })
  @ApiBody({
    description:
      'Todas las variables a excepción de id_usuario son opcionales.',
    examples: {
      ejemplo: {
        value: {
          id_usuario: 1,
          activo: true,
          multa: false,
          telefono: '',
          correo: '',
        },
      },
    },
  })
  update(@Body() body: UsuarioUpdateDto) {
    return this.usuarioService.update(body);
  }

  @Put('update-password')
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Enpoint que envía una nueva password al correo del usuario.',
  })
  @ApiBody({
    description: 'Es obligatorio enviar el campo id_usuario.',
    examples: { ejemplo: { value: { id_usuario: 1 } } },
  })
  passwordResset(@Body() body: UsuarioUpdateDto) {
    return this.usuarioService.passwordReset(body.id_usuario);
  }

  @Get('usuario')
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Enpoint que retorna la información de un usuario.',
  })
  @ApiQuery({
    description: 'El id del usuario.',
    name: 'id_usuario',
    type: 'string',
  })
  usuario(@Query() query: UsuarioDto) {
    return this.usuarioService.findById(parseInt(query.id_usuario), true, true);
  }

  @Get('usuarios')
  // @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Endpoint que retorna 25 usuarios dependiendo de la página en la que se encuentra el usuario y sus filtros.',
  })
  @ApiQuery({
    description: 'Página en la que se encuentra el usuario.',
    name: 'pagina',
    type: 'string',
  })
  @ApiQuery({
    description: 'Id de la institución que se queire usar como filtro.',
    name: 'id_institucion',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id de la carrera que se queire usar como filtro.',
    name: 'id_carrera',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Id del tipo usuario que se queire usar como filtro.',
    name: 'id_tipo_usuario',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Nombre del usuario que se queire buscar.',
    name: 'nombre',
    type: 'string',
    required: false,
  })
  @ApiQuery({
    description: 'Usuario que se queire buscar.',
    name: 'usuario',
    type: 'string',
    required: false,
  })
  usuarios(@Query() query: UsuarioUsuariosDto) {
    return this.usuarioService.findAll(query);
  }
}
